      PROGRAM FSPLTM
C     Split file with many Fortran routines into separate files.
C     This is because MS-DOS compilers do not make separate 
C     modules in the object file.
C     Language: MS-Fortran 5.1
C     Written by Robert Thrun, March 1992.
C     Not copyrighted or restricted in any way.

      PARAMETER (MAXLIN=2000)

      CHARACTER*1 TAB
      CHARACTER*40 FNAME, LIBNAM
C     If a HUGE array (Microsoft term) is larger than 128K, the element
C     size must divide evenly into 64K so that an element is not split 
C     on a segment boundary.  Isn't segmented memory wonderful?
      CHARACTER*128 LINE(MAXLIN) ! buffer to hold leading comments
      CHARACTER*72 IN1, INP      ! work images of input lines

      INTEGER*2 I, I1, I2
      INTEGER*4 NARGS          ! MS function to count cmd line args

      LOGICAL*1 OUT            ! an output file is currently open
      LOGICAL*4 FEXIST         ! a file exists

      DATA TAB /9/

C     Look at the command line for possible input file name.
C     Nonstandard Routines in Microsoft Fortran 5.0 Library:
C     NARGS() -- INT*4 function tells number of command line arguments
C        1 is executable program name, 2 and up are actual arguments
C     GETARG (numarg, string, numchar)
C        numarg -- INT*2 arg number; 0 is prog name, 1 is first arg, etc.
C        string -- character string for that arg, blank filled on right
C        numchar -- INT*2 number of characters in argument string

      IF (NARGS() .EQ. 1) GO TO 10     ! one arg is just program name
      I1 = 1
      CALL GETARG (1, FNAME, I2)       ! program name is arg(0)
      GO TO 25
   10 WRITE (*,*) 'Name of Fortran file to be split? '
      READ (*, '(A)') FNAME
C     Remove leading blanks and go to next blank after name.
      I1 = 0
   15 I1 = I1 + 1
      IF (FNAME(I1:I1) .NE. ' ') GO TO 20
      IF (I1 .LT. 40) GO TO 15
      STOP ' Blank file name'
   20 I2 = INDEX (FNAME(I1:40), ' ')
   25 INQUIRE (FILE=FNAME(I1:I2), EXIST=FEXIST)
      IF (.NOT. FEXIST) GO TO 10       ! assume name is misspelled
      OPEN (1, FILE=FNAME(I1:I2))

      OPEN (3, FILE='LIB.CMD')         ! response file to assemble lib
      I = INDEX (FNAME(I1:I2-1), '.')
      IF (I .EQ. 0) THEN
         LIBNAM = FNAME(I1:I2-1) // '.LIB'  ! name of object library
      ELSE
         LIBNAM = FNAME(I1:I1+I-2) // '.LIB'
      ENDIF
      WRITE (3, '(A / ''Y '' \)') LIBNAM

      NFILES = 0                ! number of component files
      NIN = 0                   ! number of lines read from input
   40 N = 0                     ! number of lines in buffer
      OUT = .FALSE.             ! no routine name found yet
      NOUT = 0                  ! number of lines in output file
   50 N = N + 1
      IF (N .GT. MAXLIN) THEN
         WRITE (*, '('' More than'', I5, '' leading comments'')') MAXLIN
         STOP
      ENDIF
C     We read the lines and save them because it is possible to
C     have comment lines before we find out the routine name.
      READ (1, '(A)', END=250) LINE(N)
      NIN = NIN + 1

C     Copy to work variable, expand tab in cols. 1-6, 
C     skip comments, skip continuation lines, remove trailing comments,
C     squeeze out blanks, and convert to uppercase.
      I = 0
   60 I = I + 1
      IF (LINE(N)(I:I) .EQ. TAB) GO TO 65
      IN1(I:I) = LINE(N)(I:I)
      IF (I .LT. 6) GO TO 60
      GO TO 70
   65 IN1(I:6) = '      '
   70 IN1(7:72) = LINE(N)(I+1:72)      ! IN1 now has tab expanded

C     We are interested in statements that indicate the beginning and
C     end of Fortran routines.  We don't need to examine comments,
C     continuation lines, or blank lines.  
      IF (IN1(1:1) .EQ. 'C') GO TO 80
      IF (IN1(1:1) .EQ. 'c') GO TO 80
      IF (IN1(1:1) .EQ. '*') GO TO 80
      IF (IN1(1:1) .EQ. '$') GO TO 80  ! non-ANSI comment or metacommand
      IF (INDEX(IN1(1:6),'!') .NE. 0) GO TO 80  ! comment
      IF (IN1(7:7) .EQ. ' ') GO TO 80  ! blank line
      IF (IN1(6:6) .EQ. ' ') GO TO 90  ! not continuation, may be valid
      IF (IN1(6:6) .EQ. '0') GO TO 90  ! not continuation, may be valid
C     All we have left are continuation lines and illegal statements.

C     We get here if we have a comment, blank line, continuation line,
C     or certain forms of illegal Fortran statements.
   80 IF (.NOT. OUT) GO TO 50   ! keep collecting lines in buffer

C     An output file is currently open. Write to it.
      NC = LEN_TRIM (LINE(1))   ! remove trailing blanks
      IF (NC .EQ. 0) THEN
         WRITE (2, '(A)')
      ELSE
         WRITE (2, '(A)') LINE(1)(1:NC)
      ENDIF
      NOUT = NOUT + 1           ! output file line count
      N = 0                     ! buffer of leading comments is empty
      GO TO 50                  ! next line of input

C     We get here if we have what looks like a valid statement, 
C     including Fortran routine declarations, data declarations,
C     or executable statements. 
   90 INP(1:6) = '      '       ! not used in this compressed line image
      I1 = 6
      I2 = 6
   95 I1 = I1 + 1               ! remove embedded blanks after col. 6
      IF (I1 .GT. 72) GO TO 100
      IF (IN1(I1:I1) .EQ. ' ') GO TO 95
      I2 = I2 + 1
      INP(I2:I2) = IN1(I1:I1)
      GO TO 95
  100 IF (I2 .LT. 72) INP(I2+1:72) = '   '     ! add trailing blanks

C     We have INP with embedded blanks removed.  Convert to uppercase.
      DO 110 I = 6,I2
      IF (INP(I:I) .LT. 'a') GO TO 110
      IF (INP(I:I) .GT. 'z') GO TO 110
      INP(I:I) = CHAR(ICHAR(INP(I:I))-32)
  110 CONTINUE

C     Do we have an output file name?
      IF (OUT) GO TO 200     ! write raw line image, keep looking for END

C     Look for PROGRAM, SUBROUTINE, FUNCTION, OR BLOCK DATA name
C     so that we can give a name to the output file.

      IF (I2 .LT. 14) THEN   ! too short for keyword
         FNAME = 'MAIN'      ! unnamed main program

      ELSEIF (INDEX(INP(14:I2),'=') .NE. 0) THEN ! could have PROGRAM=1.0
         FNAME = 'MAIN'

      ELSEIF (INP(7:13) .EQ. 'PROGRAM') THEN
         I2 = INDEX(INP(14:21),'(')  ! CDC programs have args
         IF (I2 .EQ. 0) THEN
            FNAME = INP(14:21)       ! max 8 char in program name
         ELSE
            FNAME = INP(14:MIN(21,I2+12))
         ENDIF
         IF (FNAME .EQ. '        ') FNAME = 'MAIN'  ! just in case

      ELSEIF (INP(7:16) .EQ. 'SUBROUTINE') THEN
C        Subroutines do not always have arguments
         I2 = INDEX(INP(17:24),'(')
         IF (I2 .EQ. 0) THEN
            FNAME = INP(17:24)       ! max 8 char in subroutine name
         ELSE
            FNAME = INP(17:MIN(24,I2+15))
         ENDIF

      ELSEIF (INP(7:15) .EQ. 'BLOCKDATA') THEN
C        Some compilers allow BLOCK DATA to have a name.
         IF (INP(16:23) .NE. '        ') THEN
            FNAME = INP(16:23)       ! max 8 char in name
         ELSE
            FNAME = 'BLOCKDAT'
         ENDIF

C     There are 19 kinds of Functions!!!
C     Maybe this should be table-driven; less code, more computation.
C     This program could make an error if the first statement of
C     an unnamed main program declares an array named 'FUNCTION'
C     FNAME always has a max of 8 chars, plus extension.

      ELSEIF (INP(7:14) .EQ. 'FUNCTION') THEN
         FNAME = INP(15:MIN(22,INDEX(INP(15:22),'(')+13))
      ELSEIF (INP(7:18) .EQ. 'BYTEFUNCTION') THEN
         FNAME = INP(19:MIN(26,INDEX(INP(19:26),'(')+17))


      ELSEIF (INP(7:21) .EQ. 'INTEGERFUNCTION') THEN
         FNAME = INP(22:MIN(29,INDEX(INP(22:29),'(')+20))
      ELSEIF (INP(7:23) .EQ. 'INTEGER*1FUNCTION') THEN
         FNAME = INP(24:MIN(31,INDEX(INP(24:31),'(')+22))
      ELSEIF (INP(7:23) .EQ. 'INTEGER*2FUNCTION') THEN
         FNAME = INP(24:MIN(31,INDEX(INP(24:31),'(')+22))
      ELSEIF (INP(7:23) .EQ. 'INTEGER*4FUNCTION') THEN
         FNAME = INP(24:MIN(31,INDEX(INP(24:31),'(')+22))

      ELSEIF (INP(7:21) .EQ. 'LOGICALFUNCTION') THEN
         FNAME = INP(22:MIN(29,INDEX(INP(22:29),'(')+20))
      ELSEIF (INP(7:23) .EQ. 'LOGICAL*1FUNCTION') THEN
         FNAME = INP(24:MIN(31,INDEX(INP(24:31),'(')+22))
      ELSEIF (INP(7:23) .EQ. 'LOGICAL*2FUNCTION') THEN
         FNAME = INP(24:MIN(31,INDEX(INP(24:31),'(')+22))
      ELSEIF (INP(7:23) .EQ. 'LOGICAL*4FUNCTION') THEN
         FNAME = INP(24:MIN(31,INDEX(INP(24:31),'(')+22))

      ELSEIF (INP(7:18) .EQ. 'REALFUNCTION') THEN
         FNAME = INP(19:MIN(26,INDEX(INP(19:26),'(')+17))
      ELSEIF (INP(7:20) .EQ. 'REAL*4FUNCTION') THEN
         FNAME = INP(21:MIN(28,INDEX(INP(21:28),'(')+19))
      ELSEIF (INP(7:20) .EQ. 'REAL*8FUNCTION') THEN
         FNAME = INP(21:MIN(28,INDEX(INP(21:28),'(')+19))

      ELSEIF (INP(7:29) .EQ. 'DOUBLEPRECISIONFUNCTION') THEN
         FNAME = INP(30:MIN(37,INDEX(INP(30:37),'(')+28))

      ELSEIF (INP(7:21) .EQ. 'COMPLEXFUNCTION') THEN
         FNAME = INP(22:MIN(29,INDEX(INP(22:29),'(')+20))
      ELSEIF (INP(7:27) .EQ. 'DOUBLECOMPLEXFUNCTION') THEN
         FNAME = INP(28:MIN(23,INDEX(INP(28:35),'(')+26))
      ELSEIF (INP(7:24) .EQ. 'COMPLEX*16FUNCTION') THEN
         FNAME = INP(25:MIN(32,INDEX(INP(25:32),'(')+23))

      ELSEIF (INP(7:23) .EQ. 'CHARACTERFUNCTION') THEN
         FNAME = INP(24:MIN(31,INDEX(INP(24:31),'(')+22))
      ELSEIF (INP(7:16) .EQ. 'CHARACTER*') THEN
         I1 = INDEX (INP(17:29),'FUNCTION') 
         IF (I1 .EQ. 0) THEN
            FNAME = 'MAIN'   ! unnamed main with CHARACTER variable
         ELSE
            I1 = I1 + 24     ! after 'FUNCTION'
            FNAME = INP(I1:MIN(I1+7,INDEX(INP(I1:I1+7),'(')+I1-2))
         ENDIF

      ELSE
         FNAME = 'MAIN'      ! unnamed main program
      ENDIF                  ! Finally!

C     Find an acceptable filename, write out the backlog of leading
C     comments (if any), and write out the current declaration line.

      NF = LEN_TRIM (FNAME)
      FNAME(NF+1:NF+4) = '.FOR'
      INQUIRE (FILE=FNAME, EXIST=FEXIST)
      IF (.NOT. FEXIST) GO TO 160
      DO 150 I = 1,999
      WRITE (FNAME(NF+2:NF+4), '(I3.3)') I    ! writes leading zeroes
      INQUIRE (FILE=FNAME, EXIST=FEXIST)
      IF (.NOT. FEXIST) GO TO 160
  150 CONTINUE

C     If the problem is more than 1000 routines that have names that
C     are identical in the first 8 characters, we could just make up
C     arbitrary names.
      WRITE (*,*)' Too many conflicting filenames: ',FNAME(1:NF+1),'*'
      STOP

  160 OPEN (2, FILE=FNAME)
      OUT = .TRUE.       ! we know routine name, output file is opened
      NFILES = NFILES + 1
C     Nonstandard '\' suppresses moving to next line.
      WRITE (*, '(I6,''. Writing '', A \ )') NFILES, FNAME(1:12)
C     Write backlog of leading comments and current line.
      DO 170 I = 1,N
      NC = LEN_TRIM (LINE(I))
      IF (NC .EQ. 0) THEN
         WRITE (2, '(A)')
      ELSE
         WRITE (2, '(A)') LINE(I)(1:NC)
      ENDIF
  170 CONTINUE
      NOUT = NOUT + N  ! count of lines in current routine
      N = 0            ! leading comments yet to be written out

C     Add to response file for Microsoft librarian.
      FNAME(NF+1:NF+4) = '.OBJ'
C     Put ' &' after file name on old line, 
C     and put '+filename.obj' on new line. '\' suppresses newline.
      WRITE (3, '('' &'' / ''+'',A \)') FNAME(1:12)

      GO TO 50   ! read next line of Fortran source file

C     We get here if we know the routine name and already 
C     have an output file and the first 6 columns are blank.
C     Look for an END statement.
  200 NC = LEN_TRIM (LINE(1))               ! remove trailing blanks
      IF (NC .EQ. 0) THEN
         WRITE (2, '(A)')
      ELSE
         WRITE (2, '(A)') LINE(1)(1:NC)
      ENDIF
      NOUT = NOUT + 1                       ! only one line in bufffer
      N = 0                                 ! flush buffer
      IF (INP(7:9) .NE. 'END') GO TO 50
C     Maybe we should test for ! comment after valid END.
      IF (INP(10:72) .NE. '    ') GO TO 50  ! could be ENDIF
      CLOSE (2)
      WRITE (*, '(I7, '' lines'')') NOUT ! same line as subroutine name
      GO TO 40   ! read next routine

C     Clean up when we hit end of input file.
  250 IF (OUT) THEN
        DO 260 I = 1,N
           NC = LEN_TRIM (LINE(I))
           IF (NC .EQ. 0) THEN
              WRITE (2, '(A)')
           ELSE
              WRITE (2, '(A)') LINE(I)(1:NC)    ! flush buffer
           ENDIF
  260   CONTINUE
        NOUT = NOUT + N
        WRITE (*, '(I7, '' lines'')') NOUT
      ENDIF
      WRITE (*, '(I8, '' lines read'', I6, '' routines'')') NIN, NFILES
      WRITE (3, '(//)')
      STOP  '  End of file'
      END

      INTEGER FUNCTION LEN_TRIM (STRING)
C     This function is not needed for Microsoft Fortran 5.0
C     This is an emulation of a Fortran-90 and MS-Fortran function.
      CHARACTER*(*) STRING
      L = LEN (STRING)
C     Find last non-blank, starting from the right end of the string.
      DO 10 I = L,1,-1
         IF (STRING(I:I) .NE. ' ') THEN
            LEN_TRIM = I
            RETURN
         ENDIF
   10 CONTINUE
      LEN_TRIM = 0
      RETURN
      END
